open Cebuffers
open Cestrings
open Cechannels
open Cegtkmisc
open Cebuffers

(* ############################################## Classe prinicpale *)
(* ------------ classe editeur : vue ------------ *)
class editeur ?(titre="(sans titre)") canaux = object (self)
  val version_camleditor = "Camleditor 2.0"
  val window = GWindow.window ~border_width:0 ()
  val langage = (GSourceView2.source_language_manager ~default:true)#language "objective-caml" 
  val tampon1 = GSourceView2.source_buffer ~highlight_matching_brackets:true ~highlight_syntax:true ()
  val tampon2 = GSourceView2.source_buffer ()
  val depuis_caml = let c,_,_ = canaux in c
  val vers_caml = let _,c,_ = canaux in c
  val mutable nbr_phr = 1
  val mutable titre = titre
  val mutable taille_fonte = 10
  
  method modifier_titre t = 
    titre <- t;
    window#set_title (version_camleditor ^ " + Caml-light : " ^ t)
  (* method avertir_erreur s =
    effacer_tag tampon2 "surligne";
    ajouter_tampon tampon2 s *)
  method code_source s =
    ajouter_tampon tampon1 s
    
  initializer
  	self#modifier_titre titre;
	let text1 = GSourceView2.source_view ~source_buffer:tampon1 ~auto_indent:true () in
	let text2 = GSourceView2.source_view ~source_buffer:tampon2 ~editable:false () in
	let statutbar = GMisc.statusbar () in 
    let contexte = statutbar#new_context ~name:"Etat"  in
    
 	(* fonctions callbacks des menus/boutons/raccourcis clavier *)
  	let quitter () = 
		print "Demande de fermeture de la fenetre"; 
		GMain.Main.quit ()
	in
	let a_propos () =
		print (version_camleditor ^ " : interface pour camllight écrite en Ocaml + lablgtk2");
		Glib.set_prgname version_camleditor;
		let fen = GWindow.about_dialog ~authors:["Laurent Walbron"] ~comments:"Interface pour Caml-light écrite en Ocaml + lablgtk2 \n " ~title:"A propos de Caml-light"  () in
		ignore (fen#run ());
		fen#destroy ()
	in
	let executer () =
	let com, debut, fin = recuperer_commande tampon1 in
	if verifie_pv com = true then begin (* on verifie si elle se termine par ;; : si oui, on envoie la commande à camllight *)
		effacer_tag tampon1 "surligne";
		write vers_caml com;
		nbr_phr <- compter_pv com;
		(* surlignage de la commande extraite *)
		appliquer_tag tampon1 "surligne" debut fin;
		ajouter_bde contexte "Commande envoyée au compilateur : en cours d'exécution";
		end
		(* sinon, on affiche un message d'erreur concernant les ;; *)
	else begin
		(* traitement de l'exception PointsVirgules *)
		ignore(GToolbox.message_box "Erreur" "Phrase incomplète : doubles points-virgules manquants.");
		(* self#avertir_erreur "Phrase incomplète : doubles points-virgules manquants.\n" *)
		end
	in
	let interrompre () =
		let commande = "/usr/bin/killall -s INT camlgraph" in (* mode graphique -> camlgraph, sinon -> camlrun *)
		ignore(Sys.command commande);
		ajouter_bde contexte "Calcul interrompu";
		print "Calcul interrompu"
	in
	let renouveler () =
    	vider_tampon tampon1;
    	vider_tampon tampon2;
    	self#modifier_titre "(sans titre)";
    	ajouter_bde contexte "Nouveau code";
	in
	let ouvrir () =
		let filew = GWindow.file_selection ~title:"Ouvrir" ~border_width:10 ~select_multiple:false () in
		let action () =
			let nom_fichier = filew#filename in
			remplir_tampon tampon1 nom_fichier;
			ajouter_bde contexte "Code chargé";
			self#modifier_titre nom_fichier;
			filew#destroy ()
		in
		ignore(filew#ok_button#connect#clicked ~callback:action);
		ignore(filew#cancel_button#connect#clicked ~callback:filew#destroy);
		filew#show ()
	in
	let enregistrer_sous () =
		let filew = GWindow.file_selection ~title:"Enregistrer sous" ~border_width:10 ~select_multiple:false () in
		let action () =
			let nom_fichier = filew#filename in
			enregistrer_tampon tampon1 nom_fichier;
			self#modifier_titre nom_fichier;
			ajouter_bde contexte ("Code enregistré" ^ titre);
			filew#destroy ()
		in
		ignore(filew#ok_button#connect#clicked ~callback:action);
		ignore(filew#cancel_button#connect#clicked ~callback:filew#destroy);
		filew#show ()		
	in
	let enregistrer () =
		if titre = "" || titre = "(sans titre)" then 
			enregistrer_sous ()
		else
			let nom_fichier = titre in
			enregistrer_tampon tampon1 nom_fichier;
			ajouter_bde contexte ("Code enregistré : " ^ titre);
	in
	let ouvrir_session_graphisme () =
		let load = "load_object \"graphics\";;" and open_module_graph = "#open \"graphics\";;" and open_graph = "open_graph \":0.0 " and taille = "500x500\";;" in
		write vers_caml load;
		write vers_caml open_module_graph;
		write vers_caml (open_graph ^ taille)
	in
	let fermer_session_graphisme () =
		let close_graph = "close_graph ();;" in
		write vers_caml close_graph
	in
	let agrandir () = (* A REVOIR : ne s'applique qu'aux éléments affichés avant et pas à ceux affichés après *)
		taille_fonte <- taille_fonte + 1;
		let taille = string_of_int taille_fonte in
		text1#misc#modify_font_by_name taille;
		text2#misc#modify_font_by_name taille;
	in
	let reduire () = (* A REVOIR : ne s'applique qu'aux éléments affichés avant et pas à ceux affichés après *)
		taille_fonte <- taille_fonte - 1;
		let taille = string_of_int taille_fonte in
		text1#misc#modify_font_by_name taille;
		text2#misc#modify_font_by_name taille;
	in
	let parametrer () =
		ignore(GToolbox.message_box "Message" "Cette fonction n'est pas encore implémentée.")
	in
  	(* fonction callback de l'événement : il existe une réponse de Caml dans la canal depuis_caml *)
	let resultat_caml list_of_conditions =
		let reponse = lire_canal depuis_caml '#' (nbr_phr) in
		(* on retire les surlignages précédents *)
		effacer_tag tampon2 "surligne";
		(* on insère le résultat en fin de tampon *)
		ajouter_tampon tampon2 reponse;
		replacer_curseur text2;
		print reponse;
		if reponse <> "\nInterrupted.\n" then ajouter_bde contexte "Commande exécutée";
		true
	in
	(* fonction callback d'interception du clavier *)
	let intercepter ev =
		let valeur_touche = (GdkEvent.Key.keyval ev) and modificateur = (GdkEvent.Key.state ev) in
		match valeur_touche, List.mem `CONTROL modificateur with
		(* Ctrl + Entrée  *) | 0xFF0D, true -> begin executer ();true end
		(* Ctrl + i *)       | 0x69, true -> begin interrompre (); true end
		(* Ctrl + s *)       | 0x73, true -> begin enregistrer (); true end
		(* Ctrl + o *)       | 0x6f, true -> begin ouvrir (); true end
		(* Ctrl + n *)       | 0x6e, true -> begin renouveler (); true end
		(* Ctrl + - *)       | 0x2d, true -> begin reduire (); true end
		(* Ctrl + + *)       | 0x3d, true -> begin agrandir (); true end
		(* autres *)         | _, _ -> begin effacer_tag tampon1 "surligne";retirer_bde contexte; false end
	in
	
  	(* construction de l'interface graphique *)
	ignore(tampon1#create_tag ~name:"surligne" [`BACKGROUND("light green")]);
	ignore(tampon2#create_tag ~name:"surligne" [`BACKGROUND("light green")]);
    tampon1#set_language (langage);
    
    ignore(window#connect#destroy ~callback:quitter);
    window#set_resizable true;
    colorier_fond (window#coerce) (255,204,56);
    
    let box_principale = GPack.vbox ~packing:(window#add) () in
	
	 let ebm = 
    [|`M ("Fichier", [`I ("Nouveau              Ctrl+N", renouveler); 
	                  `I ("Ouvrir...            Ctrl+O", ouvrir);
	                  `I ("Enregistrer          Ctrl+S", enregistrer);
	                  `I ("Enregistrer sous...        ", enregistrer_sous);
	                  `S;
	                  `I ("Quitter", quitter)
	                  ]
      );
      `M ("Compilation", [`I ("Exécuter         Ctrl+Ret", executer);
	                      `I ("Interrompre      Ctrl+I", interrompre)
	                      ]
	   );
	   `M ("Graphisme", [`I ("Ouvrir...", ouvrir_session_graphisme);
	                     `I ("Fermer", fermer_session_graphisme)
	                    ]
	   );
	   `M ("Préférences", [`I ("Agrandir        Ctrl++", agrandir);
	                       `I ("Réduire         Ctrl+-", reduire);
	                       `I ("Paramètres de l'éditeur", parametrer)
	                      ]
	   );
	   `M ("Aide", [`I ("A propos", a_propos)
	               ]
	   )
	 |] in
    (* menus *)
    let menubar = GMenu.menu_bar ~packing:(box_principale#pack ~expand:false) () in
    let create_menu label menubar =
	   let item = GMenu.menu_item ~label ~packing:menubar#append () in
	   GMenu.menu ~packing:item#set_submenu ()
	 in
    for i = 0 to (Array.length ebm - 1) do
      let `M (nom, liste_sous_menu) = ebm.(i) in
      let menu = create_menu nom menubar in
	   GToolbox.build_menu menu ~entries:liste_sous_menu;
    done;

	
    (* boîte à boutons *)
    let box_event_boutons = GBin.event_box ~packing:(box_principale#pack ~expand:false) () in

    let box_boutons = GButton.toolbar ~style:`BOTH ~packing:(box_event_boutons#add) () in
    let bouton_nouveau = GButton.tool_button ~stock:`NEW ~packing:(box_boutons#insert) () in
    let bouton_ouvrir = GButton.tool_button ~stock:`OPEN ~packing:(box_boutons#insert) () in
    let bouton_enregistrer = GButton.tool_button ~stock:`SAVE ~packing:(box_boutons#insert) () in
    let _ = GButton.separator_tool_item ~packing:(box_boutons#insert) () in 
    let bouton_compiler = GButton.tool_button ~stock:`EXECUTE ~packing:(box_boutons#insert) () in
    let bouton_interrompre = GButton.tool_button ~stock:`CANCEL ~packing:(box_boutons#insert) () in
    colorier_fond (box_event_boutons#coerce) (0,0,56);

    (* boîte pour le contrôle des marges, etc *)
    let box_padd = GBin.alignment ~padding:(0,0,1,1) ~packing:(box_principale#pack ~expand:true) () in

    (* boîte verticale pour empiler les widgets : *)
    let box = GPack.vbox ~homogeneous:true ~packing:(box_padd#add) () in	

    let box1 = GPack.vbox ~packing:(box#pack) () in
    let _ = GMisc.label ~text:"Commandes" ~justify:`CENTER ~ypad:5 ~packing:(box1#pack) () in

    let scrolled_window_haut = GBin.scrolled_window ~width:800 ~height:200 ~hpolicy:`AUTOMATIC ~vpolicy:`AUTOMATIC ~packing:(box1#pack ~expand:true) () in
    scrolled_window_haut#add(text1#coerce);

    let box2 = GPack.vbox ~packing:(box#pack) () in
    let _ = GMisc.label ~text:"Résultats" ~justify:`CENTER ~ypad:5 ~packing:(box2#pack) () in

    let scrolled_window_bas = GBin.scrolled_window ~width:800 ~height:200 ~hpolicy:`AUTOMATIC ~vpolicy:`ALWAYS ~packing:(box2#pack ~expand:true) () in
    scrolled_window_bas#add(text2#coerce);
    
    box_principale#pack(statutbar#coerce);

	 (* on attache l'événement "donnée à lire dans le canal depuis_caml" à la boucle GMain.main() *)
	 let descripteur = Unix.descr_of_in_channel depuis_caml in
	 let gtk_depuis_caml = Glib.Io.channel_of_descr descripteur in
	 ignore(Glib.Io.add_watch ~cond:[`IN] ~callback:(resultat_caml) gtk_depuis_caml);
	retirer_bde contexte;

	 
	(* connexion des signaux *)
	let _ = bouton_compiler#connect#clicked ~callback:(executer) in
	let _ = bouton_nouveau#connect#clicked ~callback:(renouveler) in
	let _ = bouton_ouvrir#connect#clicked ~callback:(ouvrir) in
	let _ = bouton_enregistrer#connect#clicked ~callback:(enregistrer)  in
	let _ = bouton_interrompre#connect#clicked ~callback:(interrompre) in
	let _ = window#event#connect#key_press ~callback:(intercepter) in

    window#show ()
end

